@@ -45,11 +45,11 @@ from Xlib import error, xauth
 uname = platform.uname()
 if (uname[0] == 'Darwin') and ([int(x) for x in uname[2].split('.')] >= [9, 0]):
 
-    display_re = re.compile(r'^([-:a-zA-Z0-9._/]*):([0-9]+)(\.([0-9]+))?$')
+    display_re = re.compile(r'^(?P<proto>)(?P<host>[-:a-zA-Z0-9._/]*):(?P<dno>[0-9]+)(\.(?P<screen>[0-9]+))?$')
 
 else:
 
-    display_re = re.compile(r'^([-:a-zA-Z0-9._]*):([0-9]+)(\.([0-9]+))?$')
+    display_re = re.compile(r'^((?P<proto>tcp|unix)/)?(?P<host>[-:a-zA-Z0-9._]*):(?P<dno>[0-9]+)(\.(?P<screen>[0-9]+))?$')
 
 def get_display(display):
     # Use $DISPLAY if display isn't provided
@@ -61,9 +61,19 @@ def get_display(display):
         raise error.DisplayNameError(display)
 
     name = display
-    host = m.group(1)
-    dno = int(m.group(2))
-    screen = m.group(4)
+    protocol, host, dno, screen = m.group('proto', 'host', 'dno', 'screen')
+    if protocol == 'tcp':
+        # Host is mandatory when protocol is TCP.
+        if not host:
+            raise error.DisplayNameError(display)
+    elif protocol == 'unix':
+        # Clear host to force Unix socket connection.
+        host = ''
+    else:
+        # Special case: `unix:0.0` is equivalent to `:0.0`.
+        if host == 'unix':
+            host = ''
+    dno = int(dno)
     if screen:
         screen = int(screen)
     else:
@@ -118,7 +128,11 @@ def new_get_auth(sock, dname, host, dno):
         family = xauth.FamilyLocal
         addr = socket.gethostname().encode()
 
-    au = xauth.Xauthority()
+    try:
+        au = xauth.Xauthority()
+    except error.XauthError:
+        return b'', b''
+
     while 1:
         try:
             return au.get_best_auth(family, addr, dno)
@@ -129,9 +143,9 @@ def new_get_auth(sock, dname, host, dno):
         # $DISPLAY to localhost:10, but stores the xauth cookie as if
         # DISPLAY was :10.  Hence, if localhost and not found, try
         # again as a Unix socket.
-        if family == xauth.FamilyInternet and addr == '\x7f\x00\x00\x01':
+        if family == xauth.FamilyInternet and addr == b'\x7f\x00\x00\x01':
             family = xauth.FamilyLocal
-            addr = socket.gethostname()
+            addr = socket.gethostname().encode()
         else:
             return b'', b''
 
